@@ -1,15 +1,17 @@
 mod build_database;
-mod console;
 mod context;
 
 use self::{build_database::BuildDatabase, context::Context};
 use crate::{
     compile::compile_dynamic,
+    console::Console,
+    debug,
     error::InfrastructureError,
     ir::{Build, Configuration, Rule},
     parse::parse_dynamic,
     utilities::read_file,
     validation::BuildGraph,
+    writeln,
 };
 use async_recursion::async_recursion;
 use futures::future::{join_all, try_join_all, FutureExt, Shared};
@@ -24,10 +26,10 @@ use std::{
 };
 use tokio::{
     fs::{create_dir_all, metadata},
-    io::{self, AsyncWriteExt},
+    io::AsyncWriteExt,
     process::Command,
     spawn,
-    sync::Semaphore,
+    sync::{Mutex, Semaphore},
     try_join,
 };
 
@@ -36,6 +38,7 @@ type BuildFuture = Shared<RawBuildFuture>;
 
 pub async fn run(
     configuration: Configuration,
+    console: &Arc<Mutex<Console>>,
     build_directory: &Path,
     job_limit: Option<usize>,
     debug: bool,
@@ -46,6 +49,7 @@ pub async fn run(
         graph,
         BuildDatabase::new(build_directory)?,
         Semaphore::new(job_limit.unwrap_or_else(num_cpus::get)),
+        console.clone(),
         debug,
     ));
 
@@ -273,13 +277,11 @@ async fn run_rule(context: &Context, rule: &Rule) -> Result<(), InfrastructureEr
         async {
             let mut console = context.console().lock().await;
 
-            if context.debug() {
-                writeln(console.stderr(), rule.command()).await?;
+            if let Some(description) = rule.description() {
+                writeln!(console.stderr(), "{}", description);
             }
 
-            if let Some(description) = rule.description() {
-                writeln(console.stderr(), description).await?;
-            }
+            debug!(context.debug(), console.stderr(), "{}", rule.command());
 
             Ok(console)
         }
@@ -289,21 +291,19 @@ async fn run_rule(context: &Context, rule: &Rule) -> Result<(), InfrastructureEr
     console.stderr().write_all(&output.stderr).await?;
 
     if !output.status.success() {
-        return Err(InfrastructureError::CommandExit(
-            rule.command().into(),
-            output.status.code(),
-        ));
+        debug!(
+            context.debug(),
+            console.stderr(),
+            "command exited{}",
+            &if let Some(code) = output.status.code() {
+                format!(" with status code {}", code)
+            } else {
+                "".into()
+            }
+        );
+
+        return Err(InfrastructureError::Build);
     }
 
     Ok(())
 }
-
-async fn writeln(
-    writer: &mut (impl AsyncWriteExt + Unpin),
-    message: impl AsRef<str>,
-) -> Result<(), io::Error> {
-    writer.write_all(message.as_ref().as_bytes()).await?;
-    writer.write_all("\n".as_bytes()).await?;
-
-    Ok(())
-}
